#!/usr/bin/python2
""">> sqlcl << command line query tool by Tamas Budavari <budavari@jhu.edu>
Usage: sqlcl [options] sqlfile(s)

Options:
        -s url	   : URL with the ASP interface (default: pha)
        -f fmt     : set output format (html,xml,csv - default: csv)
        -q query   : specify query on the command line
        -l         : skip first line of output with column names
        -v	   : verbose mode dumps settings in header
        -h	   : show this message"""

formats = ['csv','xml','html']

astro_url='http://cas.sdss.org/astro/en/tools/search/x_sql.asp'
public_url='http://cas.sdss.org/public/en/tools/search/x_sql.asp'

default_url=public_url
default_fmt='csv'

def usage(status, msg=''):
    "Error message and usage"
    print __doc__
    if msg:
        print '-- ERROR: %s' % msg
    sys.exit(status)

def filtercomment(sql):
    "Get rid of comments starting with --"
    import os
    fsql = ''
    for line in sql.split('\n'):
        fsql += line.split('--')[0] + ' ' + os.linesep;
    return fsql

def query(sql,url=default_url,fmt=default_fmt):
    "Run query and return file object"
    import urllib
    fsql = filtercomment(sql)
    params = urllib.urlencode({'cmd': fsql, 'format': fmt})
    return urllib.urlopen(url+'?%s' % params)    

def write_header(ofp,pre,url,qry):
    import  time
    ofp.write('%s SOURCE: %s\n' % (pre,url))
    ofp.write('%s TIME: %s\n' % (pre,time.asctime()))    
    ofp.write('%s QUERY:\n' % pre)
    for l in qry.split('\n'):
        ofp.write('%s   %s\n' % (pre,l))
    
def main(argv):
    "Parse command line and do it..."
    import os, getopt, string
    
    queries = []
    url = os.getenv("SQLCLURL",default_url)
    fmt = default_fmt
    writefirst = 1
    verbose = 0
    
    # Parse command line
    try:
        optlist, args = getopt.getopt(argv[1:],'s:f:q:vlh?')
    except getopt.error, e:
        usage(1,e)
        
    for o,a in optlist:
        if   o=='-s': url = a
        elif o=='-f': fmt = a
        elif o=='-q': queries.append(a)
        elif o=='-l': writefirst = 0
        elif o=='-v': verbose += 1
        else: usage(0)
        
    if fmt not in formats:
        usage(1,'Wrong format!')

    # Enqueue queries in files
    for fname in args:
        try:
            queries.append(open(fname).read())
        except IOError, e:
            usage(1,e)

    # Run all queries sequentially
    for qry in queries:
        ofp = sys.stdout
        if verbose:
            write_header(ofp,'#',url,qry)
        file = query(qry,url,fmt)
        # Output line by line (in case it's big)
        line = file.readline()
        if line.startswith("ERROR"): # SQL Statement Error -> stderr
            ofp = sys.stderr
        if writefirst:
            ofp.write(string.rstrip(line)+os.linesep)
        line = file.readline()
        while line:
            ofp.write(string.rstrip(line)+os.linesep)
            line = file.readline()


if __name__=='__main__':
    import sys
    main(sys.argv)








""" Utility functions and classes for SRP

Context : SRP
Module  : Spectroscopy.py
Version : 1.0.0
Author  : Stefano Covino
Date    : 03/09/2011
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

History : (03/09/2011) First version.

"""

import math
from SRP.SRPSpectroscopy.LymAlphaDumpConst import LymAlphaDumpConst
import SRP.SRPSpectroscopy as SS


def DLACrossSection (nu):
    num1 = 3 * SS.LambdaLymanAlpha**2 * SS.AbsOscillStrength * SS.ClassLymAlphaDumpConst
    den1 = 8 * math.pi
    num2 = LymAlphaDumpConst() * (nu/SS.NuLymanAlpha)**4
    den2 = 4 * math.pi**2 * (nu - SS.NuLymanAlpha)**2 + (LymAlphaDumpConst()**2 * (nu/SS.NuLymanAlpha)**6)/4.0
    return (num1*num2)/(den1*den2)



""" Utility functions and classes for SRP

Context : SRP
Module  : System.py
Version : 1.0.0
Author  : Stefano Covino
Date    : 27/09/2010
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks :

History : (27/09/2010) First version.

"""


import os, os.path


def Which(command):
    percorso = os.getenv("PATH")
    directories = percorso.split(os.pathsep)
    for path_dir in directories:
        real_dir = os.path.expanduser(path_dir)
        try:
            lista_dir = os.listdir(real_dir)
        except OSError:
            lista_dir = []
        if os.path.exists(real_dir) and command in lista_dir:
            return os.path.join(real_dir, command)
    return None




""" 

Context : SRP
Module  : Statistics
Version : 1.5.0
Author  : Stefano Covino
Date    : 07/06/2013
E-mail  : stefano.covino@brera.inaf.it
URL     : http://www.me.oa-brera.inaf.it/utenti/covino


Usage   : to be imported

Remarks :

History : (28/10/2010) First version.
        : (30/04/2011) ChiSqIncrement.
        : (19/02/2012) ScoreatPercentile.
        : (27/08/2012) Faster sigma clipping for frames.
        : (04/03/2013) MAD.
        : (04/04/2013) XCorr_1D.
        : (07/06/2013) FTest.
"""



__all__ = ['AsymmetricGaussianDistrib', 'AverIterSigmaClipp', 'AverSigmaClippFrame', 
           'AverSigmaClippFrameFast', 'ChSqIncrement', 'CoordDistanceMinimization', 'FTest',
           'GenFitPars', 'GenGaussSet', 'MAD', 'ScoreatPercentile', 'XCorr_1D', 'WeightedMean',
           'WeightedMeanFrame']




""" Utility functions and classes for SRP

Context : SRP
Module  : Net.py
Version : 1.0.0
Author  : Stefano Covino
Date    : 16/07/2010
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks :

History : (16/07/2010) First version.

"""

import NetConstants
import httplib


def GetWebPage (UrlAddress, UrlString):
    conn = httplib.HTTPConnection(UrlAddress)
    try:
        conn.request("GET",UrlString)
    except:
        conn.close()
        return None
    resp = conn.getresponse()
    if resp.status != NetConstants.WebQueryOK:
        conn.close()
        return None
    else:
        data = resp.read()
        conn.close()
        return data
    





""" Utility functions and classes for SRP

Context : SRP
Module  : Spectroscopy.py
Version : 1.1.0
Author  : Stefano Covino
Date    : 15/11/2013
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

History : (21/03/2011) First version.
        : (16/06/2011) Lower limit put at 0.07 micron, while in the
            original paper it was at 0.08.
        : (15/11/2013) R can be varied.

"""


import ExtConstants
from CalzettiExt import CalzettiExt
import numpy


def getAbs (ebv,glxy,lmbd, R=None):
    if not glxy in ExtConstants.ExtCurDict.keys():
        return None
    else:
        if glxy == ExtConstants.SB:
            return CalzettiExt(lmbd)*ebv
        else:
            csi = 0.0
            for i in range(len(ExtConstants.ai)):
                csi = csi + ExtConstants.ai[i][glxy]/(pow(lmbd/ExtConstants.li[i][glxy],ExtConstants.ni[i][glxy])
                    + pow(ExtConstants.li[i][glxy]/lmbd,ExtConstants.ni[i][glxy]) + ExtConstants.bi[i][glxy])
            if R == None:
                return numpy.where(lmbd < 0.07, -1.0, csi * (ebv * (1+ExtConstants.ExtRDict[glxy])))
            else:
                return numpy.where(lmbd < 0.07, -1.0, csi * (ebv * (1+R)))



""" Utility functions and classes for SRP

Context : SRP
Module  : Spectroscopy.py
Version : 1.0.0
Author  : Stefano Covino
Date    : 05/09/2011
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

History : (05/09/2011) First version.

"""


A = (-1.2150,-1.3509,-1.2150,-1.3509)
B = (1.2359,0.3786,-1.2359,-0.3786)
C = (-0.3085,0.5906,-0.3085,0.5906)
D = (0.0210,-1.1858,-0.0210,1.1858) 


def VoigtFunctApprox (a,v):
    fnz = 0.0
    for i in range(4):
        num = C[i]*(a-A[i]) + D[i]*(v-B[i])
        den = (a-A[i])**2 + (v-B[i])**2
        fnz = fnz + num/den
    return fnz




""" Utility functions and classes for SRP

Context : SRP
Module  : Math
Version : 1.0.0
Author  : Stefano Covino
Date    : 25/06/2010
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks :

History : (25/06/2010) First version.

"""

import math
import ephem



def AngularDistance (fcoord, scoord):
    ra1 = ephem.degrees(str(fcoord[0]))
    dec1 = ephem.degrees(str(fcoord[1]))
    ra2 = ephem.degrees(str(scoord[0]))
    dec2 = ephem.degrees(str(scoord[1]))
    separation = ephem.separation((ra1,dec1),(ra2,dec2))
    return math.degrees(separation)


""" Utility functions and classes for SRP

Context : SRP
Module  : Spectroscopy.py
Version : 1.0.0
Author  : Stefano Covino
Date    : 19/03/2011
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

History : (19/03/2011) First version.

"""

import math
import scipy.integrate
from VoigtIntegrand import VoigtIntegrand


def VoigtFunct (a,v):
    int = scipy.integrate.quad(VoigtIntegrand,-scipy.integrate.Inf,+scipy.integrate.Inf,args=(a,v),epsrel=1e-5)
    return (a/math.pi) * int[0]
    


""" Utility functions and classes for SRP

Context : SRP
Module  : Polarimetry
Version : 1.0.0
Author  : Stefano Covino
Date    : 21/02/2012
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks : angle is in radians
            From Witzel et al., A&A, 2011.

History : (21/02/2012) First version.

"""

import math
import numpy


def MuellerTransmissionMatrix (ordtr=1.0,extrtr=1.0):
    #
    Tp = ordtr+extrtr
    Tm = extrtr-ordtr
    Tden = Tp**2-Tm**2
    fp = Tp/Tden
    fm = Tm/Tden
    #
    r1 = [fp, -fm, 0., 0.]
    r2 = [-fm, fp, 0., 0.]
    r3 = [-fm, Tm*fm/Tp, 1./Tp, 0.]
    r4 = [-fm, Tm*fm/Tp, 0., 1./Tp]
    return 2*ordtr*extrtr*numpy.matrix([r1, r2, r3, r4])



""" Utility functions and classes for SRP

Context : SRP
Module  : Math.py
Version : 1.0.0
Author  : Stefano Covino
Date    : 03/09/2010
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks :

History : (03/09/2010) First version.

"""


import math


def CartesianCoordAmpRotoTranslation (inpcoord,shift,rotangle,ampfct):
        xold = inpcoord[0]
        yold = inpcoord[1]
        xshift = shift[0]
        yshift = shift[1]
        angle = math.radians(rotangle)
#        xnew = (xshift + xold*math.cos(angle) - yold*math.sin(angle))*ampfct
#        ynew = (yshift + xold*math.sin(angle) + yold*math.cos(angle))*ampfct
        xnew = ((xold-xshift)*math.cos(angle) - (yold-yshift)*math.sin(angle))*ampfct
        ynew = ((xold-xshift)*math.sin(angle) + (yold-yshift)*math.cos(angle))*ampfct
        return xnew,ynew


""" Utility functions and classes for SRP

Context : SRP
Module  : Catalogue.py
Version : 1.0.4
Author  : Stefano Covino
Date    : 07/01/2013
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks : 

History : (05/10/2010) First version.
        : (15/10/2010) Wrong output for SDSS filters.
        : (16/10/2010) Output better suited for skycat.
        : (05/02/2012) Better path management.
        : (07/01/2013) Better formatting for output.


"""

import os, os.path

import atpy, numpy

import SRP
from SRP.SRPMath.AstroMagInput import AstroMagInput
from SRP.SRPMath.AstroCoordInput import AstroCoordInput
from SRP.SRPMath.AngularDistance import AngularDistance
from SRP.SRPSystem.SRPPath import SRPPath



class AstroWiseStandardStars:
    basepath = SRPPath()
    filename = 'cal569E_v11.fits'
    catpath = os.path.join(basepath,'SRPData','Catalogues','AstroWiseStandardStars',filename)
    # Catalogue entries
    class CatEntries:
        def __init__ (self,(id,ra,dec,u,eu,b,eb,v,ev,r,er,i,ei,su,esu,sg,esg,sr,esr,si,esi,sz,esz)):
            # Id
            self.Id = id
            # Coord
            coord = AstroCoordInput(ra,dec)
            self.RA = coord.RA
            self.DEC = coord.DEC
            # U mag
            inpmag = AstroMagInput (u,eu)
            self.U = inpmag.Mag
            self.eU =inpmag.eMag
            # B mag
            inpmag = AstroMagInput (b,eb)
            self.B = inpmag.Mag
            self.eB =inpmag.eMag
            # V mag
            inpmag = AstroMagInput (v,ev)
            self.V = inpmag.Mag
            self.eV =inpmag.eMag
            # R mag
            inpmag = AstroMagInput (r,er)
            self.R = inpmag.Mag
            self.eR =inpmag.eMag
            # I mag
            inpmag = AstroMagInput (i,ei)
            self.I = inpmag.Mag
            self.eI =inpmag.eMag
            # u mag
            inpmag = AstroMagInput (su,esu)
            self.u = inpmag.Mag
            self.eu =inpmag.eMag
            # g mag
            inpmag = AstroMagInput (sg,esg)
            self.g = inpmag.Mag
            self.eg =inpmag.eMag
            # r mag
            inpmag = AstroMagInput (sr,esr)
            self.r = inpmag.Mag
            self.er =inpmag.eMag
            # i mag
            inpmag = AstroMagInput (si,esi)
            self.i = inpmag.Mag
            self.ei =inpmag.eMag
            # z mag
            inpmag = AstroMagInput (sz,esz)
            self.z = inpmag.Mag
            self.ez =inpmag.eMag
            # sexcoord
            self.Coord = coord

            
        def __str__ (self):
            msg = "%20s\t%15.6f\t%15.6f\t" % (self.Id, self.RA, self.DEC)
            msg = msg + "%10.3f\t%9.3f\t" % (self.U, self.eU)
            msg = msg + "%10.3f\t%9.3f\t" % (self.B, self.eB)
            msg = msg + "%10.3f\t%9.3f\t" % (self.V, self.eV)
            msg = msg + "%10.3f\t%9.3f\t" % (self.R, self.eR)
            msg = msg + "%10.3f\t%9.3f\t" % (self.I, self.eI)            
            msg = msg + "%10.3f\t%9.3f\t" % (self.u, self.eu)
            msg = msg + "%10.3f\t%9.3f\t" % (self.g, self.eg)
            msg = msg + "%10.3f\t%9.3f\t" % (self.r, self.er)
            msg = msg + "%10.3f\t%9.3f\t" % (self.i, self.ei)
            msg = msg + "%10.3f\t%9.3f" % (self.z, self.ez)
            msgo = "".join(i for i in msg if 0 < ord(i) < 128)
            return "\t".join(msgo.split())
            
            
        def __cmp__ (self, other):
            if self.V < other.V:
                return -1
            elif self.V == other.V:
                return 0
            else:
                return 1
    
    
    def __init__ (self, ra, dec, radius=1.0, epoch=2000.0):
        inpcoord = AstroCoordInput(ra,dec,inp_equinox=epoch)
        self.RA = inpcoord.RA
        self.DEC = inpcoord.DEC
        self.Radius = radius/60.0
        self.ListEntries = []
        
        
    def GetData(self):
        tb = atpy.Table(self.catpath,type='fits')
        for i in range(len(tb)):
            if AngularDistance((self.RA,self.DEC),(tb.Ra[i],tb.Dec[i])) <= self.Radius:
                datum = self.CatEntries((tb.Name[i],tb.Ra[i],tb.Dec[i],tb.JohnsonU[i],tb.JohnsonU_err[i],
                        tb.JohnsonB[i],tb.JohnsonB_err[i],tb.JohnsonV[i],tb.JohnsonV_err[i],
                        tb.CousinsR[i],tb.CousinsR_err[i],tb.CousinsI[i],tb.CousinsI_err[i],
                        tb.SloanU[i],tb.SloanU_err[i],tb.SloanG[i],tb.SloanG_err[i],
                        tb.SloanR[i],tb.SloanR_err[i],tb.SloanI[i],tb.SloanI_err[i],
                        tb.SloanZ[i],tb.SloanZ_err[i]))
                self.ListEntries.append(datum)
        return len(self.ListEntries)

    def __str__(self):
        msg = ''
        for i in self.ListEntries:
            msg = msg + str(i) + os.linesep
        return msg


    def Skycat(self, outname='SRP.cat'):
        msg = ''
        msg = msg + "long_name: SRP catalog for file %s\n" % (outname)
        msg = msg + "short_name: %s\n" % (outname)
        msg = msg + "url: ./%s\n" % (outname)
        msg = msg + "symbol: {} {circle blue} 4\n"
        msg = msg + "id_col: 0\n"
        msg = msg + "ra_col: 1\n"
        msg = msg + "dec_col: 2\n"
        msg = msg + "Id\tRA\tDEC\tJohnsonU\teJohnsonU\tJohnsonB\teJohnsonB\tJohnsonV\teJohnsonV\tCousinsR\teCousinsR\t"
        msg =  msg + "CousinsI\teCousinsI\tSloanU\teSloanU\tSloanG\teSloanG\tSloanR\teSloanR\tSloanI\teSloanI\tSloanZ\teSloanZ\n"
        msg = msg + "---------\n"
        msg = msg + str(self)
        msg = msg + "EOD\n"
        return msg
        
        
    def sort(self):
        self.ListEntries.sort()


""" Utility functions and classes for SRP

Context : SRP
Module  : Catalogue.py
Version : 1.0.7
Author  : Stefano Covino
Date    : 26/07/2013
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks :

History : (29/09/2010) First version.
        : (05/10/2010) AstroWise catalogue.
        : (24/10/2010) USNO-A2 catalogue.
        : (14/12/2010) NIR catalogue collection.
        : (28/03/2011) AGN optical standard stars.
        : (11/06/2011) USNOB1 catalogue.
        : (06/05/2013) SDSS catalogue.
        : (26/07/2013) APASS catalogue.
"""

# Available catalogues
APS = "APASS Catalogue"
BSC = "Bright Stars Catalogue"
TMC = "2MASS Catalogue"
SSC = "Stetson Optical Standard Stars"
AWC = "AstroWise Standard Stars"
USB = "USNO-B1.0 Catalogue" 
USC = "USNO-A2 Catalogue"
NCC = "NIR Catalogue Collection"
AGNOPT = "AGN Optical Standard Stars"
SDSS = "Sloan Digital Sky Survey"

SRPCatalDict = {'APS' : APS, 'BSC' : BSC, 'TMC' : TMC, 'SSC' : SSC, 'AWC' : AWC, 'USB' : USB, 'USC' : USC, 'NCC' : NCC, 'AGNOPT' : AGNOPT, 'SDSS' : SDSS}




""" Utility functions and classes for SRP

Context : SRP
Module  : Spectroscopy.py
Version : 1.0.0
Author  : Stefano Covino
Date    : 03/09/2011
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

History : (03/09/2011) First version.

"""


from SRP.SRPSpectroscopy.DLACrossSection import DLACrossSection
import SRP.SRPSpectroscopy as SS


def TauDLA (lambd,NHI,zDLA):
    return NHI * DLACrossSection((SS.SpeedofLight/lambd) * (1.0+zDLA))



""" Utility functions and classes for SRP

Context : SRP
Module  : Catalogue.py
Version : 1.0.1
Author  : Stefano Covino
Date    : 04/09/2010
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks :

History : (06/08/2010) First version.
        : (04/09/2010) New importing rules.

"""

from SRP.SRPNet.GetWebPage import GetWebPage


def GetVizierCat (QueryAddress, QueryString):
    data = GetWebPage(QueryAddress,QueryString)
    if data == None:
        return None
    retdata = []
    dataw = data.split('\n')
    for i in dataw:
        if len(i) > 0 and i[0] != '#':
            retdata.append(i)
    return retdata



""" Utility functions and classes for SRP

Context : SRP
Module  : Polarimetry
Version : 1.1.0
Author  : Stefano Covino
Date    : 09/04/2013
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks : Valid only in 0.2:1.2 micron range
            Aleksandar D. Rakic. Algorithm for the determination of intrinsic 
                optical constants of metal films: application to aluminum, Appl. 
                Opt. 34, 4755-4767 (1995)

History : (05/02/2012) First version.
        : (09/04/2013) More stable linear interpolation.

"""


import atpy, numpy
import os
from scipy.interpolate import interp1d
from SRP.SRPSystem.SRPPath import SRPPath


def AluminiumRefractiveIndex ():
    try:
        t = atpy.Table(os.path.join(SRPPath(),'SRPData','RefractiveIndex','METALS_Aluminium_Rakic.csv'), type='ascii')
    except IOError:
        return None
    #
    tw = t.where((t['lambda'] >= 0.2) & (t['lambda'] <= 1.2)) 
    tw.sort('lambda')
    #
    n1 = interp1d(tw['lambda'], tw['n'], kind='linear', bounds_error=False)
    k1 = interp1d(tw['lambda'], tw['k'], kind='linear', bounds_error=False)
    #
    return n1,k1




""" Utility functions and classes for SRP

Context : SRP
Module  : Math
Version : 1.0.0
Author  : Stefano Covino
Date    : 25/06/2010
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks : Angles in radians

History : (25/06/2010) First version.

"""

import numpy


class CartesianVector(object):
    def __init__(self, x=0.0, y=0.0, z=0.0):
        self.x = x
        self.y = y
        self.z = z

    def dot(self, v):
        return self.x * v.x + self.y * v.y + self.z * v.z

    def cross(self, v):
        n = self.__class__()
        n.x = self.y * v.z - self.z * v.y
        n.y = - (self.x * v.z - self.z * v.x)
        n.z = self.x * v.y - self.y * v.x
        return n

    @property
    def mod(self):
        return numpy.sqrt(self.x ** 2 + self.y ** 2 + self.z ** 2)

    def from_s(self, r=1.0, alpha=0.0, delta=0.0):
        self.x = r * numpy.cos(delta) * numpy.cos(alpha)
        self.y = r * numpy.cos(delta) * numpy.sin(alpha)
        self.z = r * numpy.sin(delta)
        
    def __repr__(self):
        return str(self.x, self.y, self.z)

    def __str__(self):
        self.___repr__()




"""Adapted from Practical Astronomy with your Calculator.
   By Peter Duffett-Smith with some (minor) improvements by Stefano Covino"""

from math import *

def to_julian(time):
    "Converts a time tuple to julian day"
    year,month,day,hour,min,sec=time
    #    d=day+(hour+(min+(sec/60.))/60.)/24.
    d=day+to_decimal(time[-3:])/24.0
    if month<3:
	y=year-1
	m=month+12
    else:
	y=year
	m=month
    if (year>1582 or (year==1582 and month>10) or
	(year==1582 and month==10 and day >=15)):
	A=int(y/100)
	B=2-A+int(A/4.)
    else:
	B=0
    if y<0:
	C=int((365.25*y)-0.75)
    else:
	C=int(365.25*y)
    D=int(30.6001*(m+1))
    return B+C+D+d+1720994.5


def to_calendar(day):
    "Converts a julian day to a time tuple"
    jd=0.5+day
    I=int(jd)
    F=jd-I
    if I>2229160:
	A=int((I-1867216.25)/36524.25)
	B=I+1+A-int(A/4.)
    else:
	B=I
    C=B+1524
    D=int((C-122.1)/365.25)
    E=int(365.25*D)
    G=int((C-E)/30.6001)
    d=C-E+F-int(30.6001*G)
    if G<13.5:
	month=G-1
    else:
	month=G-13
    if month>2.5:
	year=D-4716
    else:
	year=D-4715

    day=int(d)
    h=(d-day)*24
    hour,minute,sec=to_hms(h)
    return year,month,day,hour,minute,sec

def to_decimal(time):
    "Converts (hour,minute,second) format to decimal hours"
    hour,min,sec=time
    if hour<0 or min<0 or sec<0:mult=-1
    else:mult=1
    return mult*(abs(hour)+((sec/60.0)+min)/60.0)
	
def to_hms(hourd):
    "Converts decimal hours to (hour,minute,second) format"
    hour=int(hourd)
    mind=abs(hourd-hour)*60
    min=int(mind)
    sec=round((mind-min)*60,2)
    return hour,min,sec

def to_utc(time,lon,daylight=0):
    "Converts decimal localtime to utc decimal time"
    hour=time-daylight
    if lon>=0:
	zone=int(lon/15.)
    else:
	zone=int(lon/15.)-1
    hour=hour-zone
    if hour>24:
	day=1
	hour=hour-24
    elif hour<0:
	day=-1
	hour=hour+24
    else:
	day=0
    return day,hour

def to_gst(julian,utc):
    "Converts utc decima time to Greenwitch sidereal time"
    s=julian-2451545.0
    t=s/36525.0
    t0=divmod(6.697374558+(2400.051336*t)+(0.000025862*t*t),24)[1]
    gst=divmod(t0+utc*1.002737909,24)[1]
    return gst

def gst_to_utc(julian,gst):
    "Converts Greenwitch sidereal time to utc"
    t=(julian-2451545.0)/36525.0
    t0=divmod(6.697374558+(2400.051336*t)+(0.000025862*t*t),24)[1]
    ut=divmod(divmod((gst-t0),24)[1]*0.9972695663,24)[1]
    return ut

def utc_to_local(utc,lon,savings=0):
    "Converts utc to localtime"
    hour=utc+savings
    if lon>=0:
	zone=int(lon/15.)
    else:
	zone=int(lon/15.)-1
    hour=hour+zone
    if hour>24:
	day=1
	hour=hour-24
    elif hour<0:
	day=-1
	hour=hour+24
    else:
	day=0
    return day,hour

def lst_to_gst(lst,lon):
    "Converts Local sidereal time to Greenwitch sidereal time"
    return divmod(lst-lon/15.0,24)[1]

def day_of_week(julian):
    "Given Julian day, returns day of the week"
    days={0:"Sunday",1:"Monday",2:"Tuesday",3:"Wednesday",4:"Thursday",
	  5:"Friday",6:"Saturday"}
    a=(julian+1.5)/7.0
    day=int(round((a-int(a))*7.0))
    return days[day]

def rad_to_deg(rad):
    "Converts radians to degrees"
    return (rad*180.0)/pi

def deg_to_rad(deg):
    "Converts degrees to radians"
    return (deg/180.0)*pi

def eq_to_hor(h_angle,decl,lat):
    "Converts Equatorial coordinates to Horizon coordinates"
    [h_angle,decl,lat]=map(deg_to_rad,[h_angle,decl,lat])
    h_angle=h_angle*15.0
    alt=asin(sin(decl)*sin(lat)+cos(decl)*cos(lat)*cos(h_angle))
    y=-cos(decl)*cos(lat)*sin(h_angle)
    x=sin(decl)-sin(lat)*sin(alt)
    alt=rad_to_deg(alt)
    azi=atan(y/x)
    if x<0:
	azi=rad_to_deg(azi)+180
    elif x>=0 and y<0:
	azi=rad_to_deg(azi)+360
    else:
	azi=rad_to_deg(azi)
    return azi,alt

def hor_to_eq(azimuth,altitude,lat):
    "Converts Horizon coordinates to Equatorial coordinates"
    azimuth=azimuth/15.0
    h_angle,decl=eq_to_hor(azimuth,altitude,lat)
    h_angle=h_angle/15.0
    return h_angle,decl

def ecl_to_eq(ecl_lon,beta,julian=2451545.0):
    "Converts Ecliptic coordiantes to Equatorial coordiantes"
    
    ##Find the obliquity of the ecliptic for given Julian day
    t=(julian-2451545.0)/36525.0
    deps=(46.815*t+0.0006*t*t-0.00181*t*t*t)/3600.0
    eps=23.439292-deps

    [ecl_lon,beta,eps]=map(deg_to_rad,[ecl_lon,beta,eps])
    decl=asin(sin(beta)*cos(eps)+cos(beta)*sin(eps)*sin(ecl_lon))
    decl=rad_to_deg(decl)
    y=sin(ecl_lon)*cos(eps)-tan(beta)*sin(eps)
    x=cos(ecl_lon)
    ascens=atan(y/x)
    if x<0:
	ascens=rad_to_deg(ascens)+180
    elif x>=0 and y<0:
	ascens=rad_to_deg(ascens)+360
    else:
	ascens=rad_to_deg(ascens)
    ascens=ascens/15.0
    return ascens,decl

def eq_to_ecl(ascens,decl,julian=2451545.0):
    "Converts Equatorial coordinates to Ecliptic coordinates"
    ##Find the obliquity of the ecliptic for given Julian day
    t=(julian-2451545.0)/36525.0
    deps=(46.815*t+0.0006*t*t-0.00181*t*t*t)/3600.0
    eps=23.439292-deps

    ascens=ascens*15.0
    [ascens,decl,eps]=map(deg_to_rad,[ascens,decl,eps])
    beta=asin(sin(decl)*cos(eps)-cos(decl)*sin(eps)*sin(ascens))
    beta=rad_to_deg(beta)
    y=sin(ascens)*cos(eps)+tan(decl)*sin(eps)
    x=cos(ascens)
    ecl_lon=atan(y/x)
    if x<0:
	ecl_lon=rad_to_deg(ecl_lon)+180
    elif x>=0 and y<0:
	ecl_lon=rad_to_deg(ecl_lon)+360
    else:
	ecl_lon=rad_to_deg(ecl_lon)
    return ecl_lon,beta
    
def solve_kepler(ecc,mean_anom):
    "Solves Kepler equation iteratively"
    e=mean_anom
    d=e-ecc*sin(e)-mean_anom
    while d>0.00000001:
	de=d/(1-ecc*cos(e))
	e=e-de
	d=e-ecc*sin(e)-mean_anom
    return e
    
def sun(julian):
    "Calculates Sun's ecliptic longitude"
    T=(julian-2415020)/36525.0
    ##    eg=279.6966778+36000.76892*T+0.0003025*T*T
    eg=279.403303
    wg=281.2208444+1.719175*T+0.000452778*T*T
    e=0.01675104-0.0000418*T-0.000000126*T*T
    days=julian-2447891.5
    n=divmod(360*days/365.242191,360)[1]
#    if n<0:n=n+360
    m=n+eg-wg
    if m<0:m=m+360
    E=solve_kepler(e,deg_to_rad(m))
    niou=rad_to_deg(2*atan(sqrt((1+e)/(1-e))*tan(E/2.0)))
    l=divmod(niou+wg,360)[1]
    if l<0:l=l+360
    return l

def planet(julian,plan_elem,earth_elem):
    "Calculate orbital parameters of a planet"
    (t_p,eps_p,w_p,e_p,a_p,i,omega)=plan_elem
    (t_e,eps_e,w_e,e_e,a_e,tmp1,tmp2)=earth_elem
    # For planet
    d=julian-2447891.5
    n_p=(360/365.242191)*(d/t_p)%360
    m_p=n_p+eps_p-w_p
    l=(n_p+(360/pi)*e_p*sin(deg_to_rad(m_p))+eps_p)%360
    niou_p=l-w_p
    r=a_p*(1-e_p*e_p)/(1+e_p*cos(deg_to_rad(niou_p)))
    ## For earth
    n_e=(360/365.242191)*(d/t_e)%360
    m_e=n_e+eps_e-w_e
    L=(n_e+(360/pi)*e_e*sin(deg_to_rad(m_e))+eps_e)%360
    niou_e=l-w_e
    R=a_e*(1-e_e*e_e)/(1+e_e*cos(deg_to_rad(niou_e)))
    psi=asin(sin(deg_to_rad(l-omega))*sin(deg_to_rad(i)))
    y=sin(deg_to_rad(l-omega))*cos(deg_to_rad(i))
    x=cos(deg_to_rad(l-omega))
    dl=atan(y/x)
    if x<0:
	dl=rad_to_deg(dl)+180
    elif x>=0 and y<0:
	dl=rad_to_deg(dl)+360
    else:
	dl=rad_to_deg(dl)
    l_dash=dl+omega
    r_dash=r*cos(psi)
    if a_p<1.0:
	A=rad_to_deg(atan((r_dash*sin(deg_to_rad(L-l_dash)))
			  /(R-r_dash*cos(deg_to_rad(L-l_dash)))))
	ecl_lon=(180+L+A)%360
    else:
	ecl_lon=(rad_to_deg(atan((R*sin(deg_to_rad(l_dash-L)))
	       /(r_dash-R*cos(deg_to_rad(l_dash-L)))))+l_dash)%360
    beta=rad_to_deg(atan((r_dash*tan(psi)*sin(deg_to_rad(ecl_lon-l_dash)))
			     /(R*sin(deg_to_rad(l_dash-L)))))
    return ecl_lon,beta




""" Utility functions and classes for SRP

Context : SRP
Module  : Math.py
Version : 1.0.0
Author  : Stefano Covino
Date    : 11/02/2012
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks :

History : (11/02/2012) First version.

"""

import math

from SRP.SRPMath.PointClass import Point




def PointMatch (reflist,objlist,tol=5.0,shx=0.0,shy=0.0,force=False):
#    print objlist
#    print reflist
    reqmtch = int(math.ceil(len(reflist)/2.))
    #
    for i in range(len(reflist)):
        for l in range(len(objlist)):
            #print i,l
            pref = Point(reflist[i])
            pobj = Point(objlist[l])
            distx, disty = pref.ShiftDist(pobj)
            #
            if force:
                distx, disty = shx, shy
            #
            assobj = []
            for ii in range(len(reflist)):
                for ll in range(len(objlist)):
                    if Point(reflist[ii],distx,disty).PointDist(Point(objlist[ll])) <= tol:
                        assobj.append((ii,ll))
                        break
            if len(assobj) >= reqmtch:
                return assobj, (distx, disty)
    #
    return [], (0.0,0.0)


""" Utility functions and classes for SRP

Context : SRP
Module  : Math.py
Version : 1.0.1
Author  : Stefano Covino
Date    : 21/03/2012
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks :

History : (02/09/2010) First version.
        : (21/03/2012) Better management of math errors.

"""

import math

from PointClass import Point
from AstroAngleInput import AstroAngleInput
from CartesianCoordAmpRotoTranslation import CartesianCoordAmpRotoTranslation
from InverseCartesianCoordAmpRotoTranslation import InverseCartesianCoordAmpRotoTranslation

class Triangle:
    def __init__ (self, v0, v1, v2, atol=1.0):
        self.Vertici = [Point(v0), Point(v1), Point(v2)]
        self.Sizes = [self.Vertici[1].PointDist(self.Vertici[2]),
            self.Vertici[0].PointDist(self.Vertici[2]),
                self.Vertici[1].PointDist(self.Vertici[0])]
        try:
            self.Angles = [math.degrees(math.acos((self.Sizes[2]**2 + self.Sizes[1]**2 - self.Sizes[0]**2)/(2*self.Sizes[2]*self.Sizes[1]))),
                        math.degrees(math.acos((self.Sizes[0]**2 + self.Sizes[2]**2 - self.Sizes[1]**2)/(2*self.Sizes[0]*self.Sizes[2]))),
                            math.degrees(math.acos((self.Sizes[1]**2 + self.Sizes[0]**2 - self.Sizes[2]**2)/(2*self.Sizes[1]*self.Sizes[0])))]
        except ValueError:
            self.Angles = [0,0,0]
        except ZeroDivisionError:
            self.Angles = [0,0,0]
        self.AngTol = AstroAngleInput(atol).Angle
        self.ConnVert = []


    def IsSymmTriangle (self):
        neqangl = 0
        if math.fabs(self.Angles[0] - self.Angles[1]) <= self.AngTol:
            neqangl = neqangl + 1
        if math.fabs(self.Angles[1] - self.Angles[2]) <= self.AngTol:
            neqangl = neqangl + 1
        if math.fabs(self.Angles[2] - self.Angles[0]) <= self.AngTol:
            neqangl = neqangl + 1
        if neqangl >= 1:
            return True
        else:
            return False

    
    def Commensurabili(self,other):
        eqang = []
        assangl = [False for i in range(3)]
        for i in range(3):
            for l in range(3):
                if (assangl[l] == False) and math.fabs(self.Angles[i] - other.Angles[l]) <= self.AngTol:
#                    print self.Angles[i],other.Angles[l]
                    eqang.append((i,l))
                    assangl[l] = True
                    break
        self.ConnVert = eqang 
        if len(eqang) < 3:
            return False
        else:            
            return True
            
    
    def Rotable (self, other):
        if len(self.ConnVert) !=3:
            self.Rot = 0.0
            return False
        unoa = self.Vertici[self.ConnVert[0][0]].AngOrig(self.Vertici[self.ConnVert[1][0]].Coord)
        duea = other.Vertici[self.ConnVert[0][1]].AngOrig(other.Vertici[self.ConnVert[1][1]].Coord)
        rota = duea-unoa
#        print duea,unoa,rota
        unob = self.Vertici[self.ConnVert[0][0]].AngOrig(self.Vertici[self.ConnVert[2][0]].Coord)
        dueb = other.Vertici[self.ConnVert[0][1]].AngOrig(other.Vertici[self.ConnVert[2][1]].Coord)
        rotb = dueb-unob
#        print dueb,unob,rotb
        unoc = self.Vertici[self.ConnVert[1][0]].AngOrig(self.Vertici[self.ConnVert[2][0]].Coord)
        duec = other.Vertici[self.ConnVert[1][1]].AngOrig(other.Vertici[self.ConnVert[2][1]].Coord)
        rotc = duec-unoc
#        print duec,unoc,rotc
        if math.fabs(rota-rotb) > self.AngTol or math.fabs(rota-rotc) > self.AngTol or math.fabs(rotb-rotc) > self.AngTol:
            return False
        else:
            return True
            
                
    def SizeFactor (self,other):
        sisfact = 0.0
        if len(self.ConnVert) != 3:
            self.SiFa = 1.0
            return self.SiFa
        else:
            for p in self.ConnVert:
                sisfact = sisfact + self.Sizes[p[0]]/other.Sizes[p[1]]
#                print self.Sizes[p[0]],other.Sizes[p[1]]
            self.SiFa = sisfact/3.0
            return self.SiFa
        
    def RotationFactor (self,other):
        if len(self.ConnVert) !=3:
            self.Rot = 0.0
            return self.Rot
        unoa = self.Vertici[self.ConnVert[0][0]].AngOrig(self.Vertici[self.ConnVert[1][0]].Coord)
        duea = other.Vertici[self.ConnVert[0][1]].AngOrig(other.Vertici[self.ConnVert[1][1]].Coord)
        rota = duea-unoa
#        print duea,unoa,rota
        unob = self.Vertici[self.ConnVert[0][0]].AngOrig(self.Vertici[self.ConnVert[2][0]].Coord)
        dueb = other.Vertici[self.ConnVert[0][1]].AngOrig(other.Vertici[self.ConnVert[2][1]].Coord)
        rotb = dueb-unob
#        print dueb,unob,rotb
        unoc = self.Vertici[self.ConnVert[1][0]].AngOrig(self.Vertici[self.ConnVert[2][0]].Coord)
        duec = other.Vertici[self.ConnVert[1][1]].AngOrig(other.Vertici[self.ConnVert[2][1]].Coord)
        rotc = duec-unoc
#        print duec,unoc,rotc
        self.Rot = (rota+rotb+rotc)/3.0
#        print "rot",rota,rotb,rotc
        return self.Rot

    
    def ShiftFactor (self,other):
        if self.SiFa == None:
            self.SiFa = 1.0
        if self.Rot == None:
            self.Rot = 0.0
        newc = CartesianCoordAmpRotoTranslation(self.Vertici[self.ConnVert[0][0]].Coord,(0,0),self.Rot,1./self.SiFa)
        sha = (other.Vertici[self.ConnVert[0][1]].Coord[0]-newc[0]),(other.Vertici[self.ConnVert[0][1]].Coord[1]-newc[1])
        newc = CartesianCoordAmpRotoTranslation(self.Vertici[self.ConnVert[1][0]].Coord,(0,0),self.Rot,1./self.SiFa)
        shb = (other.Vertici[self.ConnVert[1][1]].Coord[0]-newc[0]),(other.Vertici[self.ConnVert[1][1]].Coord[1]-newc[1])
        newc = CartesianCoordAmpRotoTranslation(self.Vertici[self.ConnVert[2][0]].Coord,(0,0),self.Rot,1./self.SiFa)
        shc = (other.Vertici[self.ConnVert[2][1]].Coord[0]-newc[0]),(other.Vertici[self.ConnVert[2][1]].Coord[1]-newc[1])  
#        print "shift",sha,shb,shc
        ShFa = (sha[0]+shb[0]+shc[0])/(3.0*(-1./self.SiFa)),(sha[1]+shb[1]+shc[1])/(3.0*(-1./self.SiFa)) 
#        print "shift",ShFa
        TrueSh = InverseCartesianCoordAmpRotoTranslation(ShFa,(0,0),self.Rot,1.)
#        print "tshift", TrueSh
        self.ShFa = TrueSh      
        return self.ShFa
        



from ez_setup import use_setuptools
use_setuptools()

from setuptools import setup, find_packages 
import glob, os.path, sys

# Read long description
def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()


# Path check
def which(command):
    percorso = os.getenv("PATH")
    directories = percorso.split(os.pathsep)
    for path_dir in directories:
        real_dir = os.path.expanduser(path_dir)
        try:
            lista_dir = os.listdir(real_dir)
        except OSError:
            lista_dir = []
        if os.path.exists(real_dir) and command in lista_dir:
            return os.path.join(real_dir, command)
    return None


# Look for script files
lscr = glob.glob(os.path.join('Scripts', 'SRP*'))
lscrex = []
for i in lscr:
    if os.path.splitext(i)[1] == '':
        lscrex.append(i)


# python
print "Python version %s" % (sys.version)
pvl = sys.version.split()[0].split('.')
if int(pvl[0]) != 2 or int(pvl[1]) < 3:
    print "Python version should be 2.x with x at least 6."


import SRP
durl = 'http://www.me.oa-brera.inaf.it/utenti/covino/SRPAstro-%s.tar.gz' % SRP.__version__


setup(
    name='SRPAstro', 
    version=SRP.__version__, 
    description='Data Analysis Packages', 
    packages = find_packages('.'),
    include_package_data = True,
    long_description='Set of tools for carrying out simple tasks related to astronomical observations',
    author='Stefano Covino', 
    author_email='stefano.covino@brera.inaf.it', 
    url='http://www.me.oa-brera.inaf.it/utenti/covino/SRPAstro.pdf', 
    download_url=durl,    
    install_requires=['astropy', 'scipy', 'astlib>=0.4', 'pil', 'matplotlib', 'atpy',
        'asciitable', 'pyfits', 'pyephem', 'numpy>=1.1'],
    scripts=lscrex,
    zip_safe = False,
    package_data={'SRPAstro':['SRPCalFlux.data','SRPData/*']},
    classifiers=[ 
        'Development Status :: 5 - Production/Stable', 
        'Environment :: Console', 
        'Intended Audience :: Science/Research', 
        'License :: Freely Distributable', 
        'Operating System :: MacOS :: MacOS X', 
        'Operating System :: POSIX', 
        'Programming Language :: Python :: 2', 
        'Topic :: Scientific/Engineering :: Astronomy', 
        ], 
    ) 


# nose



""" Utility functions and classes for SRP

Context : SRP
Module  : Spectroscopy.py
Version : 1.0.0
Author  : Stefano Covino
Date    : 29/06/2011
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : SolarAbundance(element=None)
            the script retrieves the solar abundance according to the table by Asplund et al. (2009)

History : (29/06/2011) First version.

"""

import SRP.SRPSpectroscopy


def SolarAbundance (element=None):
    if element == None:
        return SRP.SRPSpectroscopy.SolAbDict.keys()
    else:
        try:
            return SRP.SRPSpectroscopy.SolAbDict[element]
        except KeyError:
            return None



""" Utility functions and classes for SRP

Context : SRP
Module  : Math
Version : 1.0.0
Author  : Stefano Covino
Date    : 25/06/2010
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks :

History : (25/06/2010) First version.

"""

import numpy
from CartesianVectorClass import CartesianVector


def FastAngularDistance (a1,l1,a2,l2):
    v = CartesianVector()
    v.from_s(1.0, numpy.radians(a1), numpy.radians(l1))
    v2 = CartesianVector()
    v2.from_s(1.0, numpy.radians(a2), numpy.radians(l2))
    d = v.dot(v2)
    c = v.cross(v2).mod

    res = numpy.arctan2(c, d)

    return numpy.degrees(res)




""" Utility functions and classes for SRP

Context : SRP
Module  : SRPUtil.py
Version : 1.22.1
Status  : approved
Author  : Stefano Covino, Nino Cucchiara
Date    : 11/03/2014
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino
Purpose : Collection of utility functions and classes for SRP.

Usage   : to be imported

Remarks :

History : (29/05/2003) First version.
        : (15/06/2003) Better good range.
        : (09/09/2003) Magnitude to/from flux.
        : (02/10/2003) Reddening.
        : (07/10/2003) Angular distance.
        : (14/10/2003) Better SRPPhotometry output.
        : (23/12/2003) Zero-point in star data.
        : (01/09/2003) Rad to deg and viceversa functions.
        : (26/10/2004) DAOPHOT data classes.
        : (25/12/2004) Sigma clipping function.
        : (03/07/2005) Calzetti's Extinction Law
        : (13/04/2006) Bug corrected in which.
        : (25/02/2007) Absorption at the X-rays.
        : (15/11/2007) Corrections to absorption curves.
        : (21/11/2007) Star Data class for GAIA photom output.
        : (14/12/2007) Better management of redshifted data for X-ray absorption.
        : (12/10/2008) Peak data.
        : (18/10/2008) angleRange.
        : (22/10/2008) Rototraslations.
        : (16/11/2008) Better mangament of sigma-clipping algorithms.
        : (18/04/2009) Jansky to erg/scm2A conversion.
        : (21/05/2010) Better FITS header mamagement.
        : (17/06/2010) Better data file path management
        : (22/06/2010) Minor improvement.
        : (30/09/2010) Stardata improvement.
        : (14/10/2010) Minor correction in Stardata.
        : (21/03/2011) Better coding for extinction routines.
        : (20/08/2011) Exptime in StarData.
        : (23/10/2011) No more Jy 2 ergs/s cm2 A and viceversa conversion.
        : (30/10/2011) Ellipticity added to StarData class.
        : (18/12/2012) Airmass in Daophot and GAIA output.
        : (11/02/2013) Classification added to StarData class.
        : (11/03/2014) Bug in daophot allstar output.
"""


import SRPFiles, SRPConstants
import os, string, math, copy
import TimeAstro_algs
import SRP.stats as stats




#def which(command):
#    percorso = os.getenv("PATH")
#    directories = percorso.split(os.pathsep)
#    for path_dir in directories:
#        real_dir = os.path.expanduser(path_dir)
#        try:
#            lista_dir = os.listdir(real_dir)
#        except OSError:
#            lista_dir = []
#        if os.path.exists(real_dir) and command in lista_dir:
#            return os.path.join(real_dir, command)
#    return None



class StarData:
    def __init__ (self, dlista, exptime, zpl=(25.0, 0.0), mjd=-99.0, arm=1.0, flt='Unknown'):
        self.Id = dlista[0]
        self.X = float(dlista[1])
        self.Y = float(dlista[2])
        self.RA = float(dlista[3])
        self.DEC = float(dlista[4])
        self.magap = float(dlista[5])
        self.emagap = float(dlista[6])
        self.mag = float(dlista[7])
        self.emag = float(dlista[8])
        self.sky = float(dlista[9])
        self.fmax = float(dlista[10])
        self.FWHM = float(dlista[11])
        self.Ellipticity = float(dlista[12])
        self.Classification = float(dlista[13])
        self.mag = self.mag + 2.5*math.log10(exptime)
        self.magap = self.magap + 2.5*math.log10(exptime)
        self.magap = self.magap + zpl[0] - 25.0
        self.mag = self.mag + zpl[0] - 25.0
        self.emagap = math.sqrt(math.pow(self.emagap,2)+math.pow(zpl[1],2))
        self.emag = math.sqrt(math.pow(self.emag,2)+math.pow(zpl[1],2))
        self.MJD = mjd
        self.Airmass = float(arm)
        self.Filter = flt
        self.Exptime = exptime

    def __str__ (self):
        ostr = "%10s\t%7.2f\t%7.2f\t%10.6f\t%10.6f\t" % (self.Id, self.X, self.Y, self.RA, self.DEC)
        ostr = ostr + "%7.3f\t%6.3f\t%9.3f\t%15.3f\t" % (self.magap, self.emagap, self.sky, self.fmax)
        ostr = ostr + "%7.3f\t%6.3f\t%5.2f\t%15s\t" % (self.mag, self.emag, self.FWHM, self.MJD)
        ostr = ostr + "%7.3f\t%15s\t%7.2f\t%7.2f\t" % (self.Airmass, self.Filter, self.Exptime, self.Ellipticity)
        ostr = ostr + "%5.2f" % (self.Classification)
        return ostr+os.linesep



def getStarData (dlista,exptime, zpl=(25.0,0.0), mjd=-99.0, arm=1.0, flt='Unknown'):
    slista = []
    for i in range(len(dlista)):
        if dlista[i] != '':
            l = string.split(dlista[i])
            if len(l) > 0 and l[0] != '>' and len(l) == 14:
                s = StarData(l,exptime,zpl,mjd,arm,flt)
                slista.append(s)
    return slista



def getGoodRange (range,fact):
    zone = []
    xmin = int(1 + (range[1]-range[0]) * fact/100.0)
    xmax = int(range[1] -  (range[1]-range[0]) * fact/100.0)
    if len(range) == 4:
        ymin = int(1 + (range[3]-range[2]) * fact/100.0)
        ymax = int(range[3] - (range[3]-range[2]) * fact/100.0)
        zone = [xmin,xmax,ymin,ymax]
    else:
        zone = [xmin,xmax]
    return zone



def getRange (header):
    values = []
    naxis = header['NAXIS']
    xmin = 1
    xmax = header["NAXIS1"]
    if xmax == '':
        xmax = 1
    if naxis >= 2:
        ymin = 1
        ymax = header["NAXIS2"]
        if ymax == '':
            ymax = 1
        values = [xmin,xmax,ymin,ymax]
    else:
        values = [xmin,xmax]
    return values


# Fluxes and magnitudes
def getMagnitude (fl,efl,ban):
    sp = SRPFiles.getSRPDataPath()
    if sp == None:
        nofi = SRPConstants.SRPMagFluxFile
    else:
        nofi = os.path.join(sp,SRPConstants.SRPMagFluxFile)
    band, l0, f0, n0 = SRPFiles.SRPLeggiMagCalFile (nofi)
    #Cerchiamo la banda
    mag, emag, le = SRPConstants.SRPMagErr, SRPConstants.SRPMagErr, SRPConstants.SRPMagErr
    for i in range(len(band)):
        if string.strip(string.upper(band[i])) == string.strip(string.upper(ban)):
            mag = -2.5*(math.log10(fl/f0[i]))
            emag = (2.5/math.log(10.0))*efl/fl
            le = l0[i]
    return mag, emag, le


def getFlux (mag,emag,ban):
    sp = SRPFiles.getSRPDataPath()
    if sp == None:
        nofi = SRPConstants.SRPMagFluxFile
    else:
        nofi = os.path.join(sp,SRPConstants.SRPMagFluxFile)
    band, l0, f0, n0 = SRPFiles.SRPLeggiMagCalFile (nofi)
    #Cerchiamo la banda
    fl, efl, ne = SRPConstants.SRPMagErr, SRPConstants.SRPMagErr, SRPConstants.SRPMagErr
    for i in range(len(band)):
        if string.strip(string.upper(band[i])) == string.strip(string.upper(ban)):
            fl = f0[i]*math.pow(10,(-mag/2.5))
            efl = emag*fl/(2.5/math.log(10.0))
            ne = n0[i]
    return fl, efl, ne


def getBand ():
    sp = SRPFiles.getSRPDataPath()
    if sp == None:
        nofi = SRPConstants.SRPMagFluxFile
    else:
        nofi = os.path.join(sp,SRPConstants.SRPMagFluxFile)
    band, l0, f0, n0 = SRPFiles.SRPLeggiMagCalFile (nofi)
    return band




# Angular distance
def AngDistance(pos1, pos2):
    p1a = TimeAstro_algs.deg_to_rad(pos1[0])
    p1b = TimeAstro_algs.deg_to_rad(pos1[1])
    p2a = TimeAstro_algs.deg_to_rad(pos2[0])
    p2b = TimeAstro_algs.deg_to_rad(pos2[1])
    A = math.sin(p1b) * math.sin(p2b)
    B = math.cos(p1b) * math.cos(p2b)
    C = math.cos(p1a-p2a)
    D = TimeAstro_algs.rad_to_deg(math.acos(A+B*C))
    return D




# Calzetti's Extinction Law
#def CalzettiExt(l,R=4.05):
#    if l < 0.12:
#        return 0.0
#    elif 0.12 <= l < 0.63:
#        return 2.659*(-2.156 + 1.509/l - 0.198/l**2 + 0.011/l**3) + R
#    elif 0.63 <= l <= 2.2:
#        return 2.659*(-1.857 + 1.040/l) + R
#    else:
#        return 1.0







# Deg to rad and viceversa
def deg2rad(ang):
    return ang*math.pi/180.0


def rad2deg(ang):
    return ang*180.0/math.pi



# DAOPHOT data

class DAOStarApData:
    def __init__ (self, exptime, n, x, y, maglist, emaglist, sky, esky, skew, zp, am):
        self.Id = int(n)
        self.X = float(x)
        self.Y = float(y)
        self.MagList = []
        self.eMagList = []
        for i in range(len(maglist)):
            self.MagList.append(float(maglist[i])+2.5*math.log10(exptime)+zp[0])
            er = math.sqrt(math.pow(float(emaglist[i]),2)+math.pow(zp[1],2))
            self.eMagList.append(er)
        self.Sky = float(sky)
        self.eSky = float(esky)
        self.SSkew = float(skew)
        self.Airmass = float(am)

    def __str__ (self):
        msg = ''
        msg = msg + "%6d\t%10.3f\t%10.3f" % (self.Id, self.X, self.Y)
        msg = msg + "\t%10.2f\t%10.2f\t%10.2f" % (self.Sky, self.eSky, self.SSkew)
        for i in range(len(self.MagList)):
            msg = msg + "\t%8.3f\t%8.3f" % (self.MagList[i], self.eMagList[i])
        msg = msg + "\t%8.3f" % (self.Airmass)
        return msg



class DAOStarPsfData:
    def __init__ (self, exptime, n, x, y, sky, it, chi, sharp, mag, emag, zp, am):
        self.Id = int(n)
        self.X = float(x)
        self.Y = float(y)
        self.Mag = float(mag)+2.5*math.log10(exptime)+zp[0]
        self.eMag = math.sqrt(math.pow(float(emag),2)+math.pow(zp[1],2))
        self.Sky = float(sky)
        self.nIt = int(float(it))
        self.Chi = float(chi)
        self.Sharp = float(sharp)
        self.Airmass = float(am)

    def __str__ (self):
        msg = ''
        msg = msg + "%6d\t%10.3f\t%10.3f" % (self.Id, self.X, self.Y)
        msg = msg + "\t%10.2f\t%4d\t%5.1f" % (self.Sky, self.nIt, self.Chi)
        msg = msg + "\t%5.1f\t%8.3f\t%8.3f" % (self.Sharp, self.Mag, self.eMag)
        msg = msg + "\t%8.3f" % (self.Airmass)
        return msg



# Average sigma clipping
#def averageSigmaClippingIter (inplst, siglev = 2.0):
#       wlst = copy.copy(inplst)
#       card = len(wlst)
#       while True:
#               if len(wlst) < 1:
#                        return None,None,None
#                elif len(wlst) == 1:
#                        return wlst[0],0.0,0.0
#               wa = stats.mean(wlst)
#               ws = stats.samplestdev(wlst)
#               we = ws/math.sqrt(len(wlst))
#               wmin, wmax = wa - siglev*ws, wa + siglev*ws
#               print wmin, wmax, len(wlst), wa, ws, we
#               for i in wlst:
#                       if not wmin <= i <= wmax:
#                               wlst.remove(i)
#               if card <> len(wlst):
#                       card = len(wlst)
#               else:
#                       return wa,ws,we


#def averageSigmaClipping (inplst, siglev = 2.0):
#        if len(inplst) < 1:
#               return None,None,None
#        elif len(inplst) == 1:
#                return inplst[0],0.0,0.0
#       wa = stats.mean(inplst)
#       ws = stats.samplestdev(inplst)
#       we = ws/math.sqrt(len(inplst))
#       wmin, wmax = wa - siglev*ws, wa + siglev*ws
#        wlst = []
#       for i in inplst:
#               if wmin <= i <= wmax:
#                       wlst.append(i)
#        if len(wlst) == 1:
#                return wlst[0],0.0,0.0
#       wa = stats.mean(wlst)
#       ws = stats.samplestdev(wlst)
#       we = ws/math.sqrt(len(wlst))
#        return wa,ws,we



# X-ray absorption
def XAbsorption (energy, nh=1e20, z=0):
    ren = energy * (1 + z)
    if ren < 0.03:
        return 0.0
    elif ren > 10.0:
        return 1.0
    if nh < 0:
        return 0.0
    if 0.03 <= ren < 0.1:
        C1 = 17.3
        C2 = 608.1
        C3 = -2150.0
    elif 0.1 <= ren < 0.284:
        C1 = 34.6
        C2 = 267.9
        C3 = -476.1
    elif 0.284 <= ren < 0.400:
        C1 = 78.1
        C2 = 18.8
        C3 = 4.3
    elif 0.4 <= ren < 0.532:
        C1 = 71.4
        C2 = 66.8
        C3 = -51.4
    elif 0.532 <= ren < 0.707:
        C1 = 95.5
        C2 = 145.8
        C3 = -61.1
    elif 0.707 <= ren < 0.867:
        C1 = 308.9
        C2 = -380.6
        C3 = 294.0
    elif 0.867 <= ren < 1.303:
        C1 = 120.6
        C2 = 169.3
        C3 = -47.7
    elif 1.303 <= ren < 1.840:
        C1 = 141.3
        C2 = 146.8
        C3 = -31.5
    elif 1.840 <= ren < 2.471:
        C1 = 202.7
        C2 = 104.7
        C3 = -17.0
    elif 2.471 <= ren < 3.210:
        C1 = 342.7
        C2 = 18.7
        C3 = 0.0
    elif 3.210 <= ren < 4.038:
        C1 = 352.2
        C2 = 18.7
        C3 = 0.0
    elif 4.038 <= ren < 7.111:
        C1 = 433.9
        C2 = -2.4
        C3 = 0.75
    elif 7.111 <= ren < 8.331:
        C1 = 629.0
        C2 = 30.9
        C3 = 0.0
    elif ren >= 8.331:
        C1 = 701.2
        C2 = 25.2
        C3 = 0.0
    xcs = 1e-24*(C1 + C2*ren + C3*ren**2) / ren**3
    return math.e**-(xcs*nh)




# GAIA data

class GAIAStarData:
    def __init__ (self, exptime, n, x, y, mag, emag, sky, totsignal, radius, zp, am):
        self.Id = int(n)
        self.X = float(x)
        self.Y = float(y)
        self.Mag = float(mag)+2.5*math.log10(exptime)+zp[0]
        self.eMag = math.sqrt(math.pow(float(emag),2)+math.pow(zp[1],2))
        self.Sky = float(sky)
        self.TotSignal = float(totsignal)
        self.Radius = [float(radius[0]), float(radius[1])*float(radius[0]), float(radius[2])*float(radius[0])]
        self.Airmass = am

    def __str__ (self):
        msg = ''
        msg = msg + "%6d\t%10.3f\t%10.3f" % (self.Id, self.X, self.Y)
        msg = msg + "\t%8.3f\t%8.3f" % (self.Mag, self.eMag)
        msg = msg + "\t%10.2f\t%10g" % (self.Sky, self.TotSignal)
        msg = msg + "\t%6.2f\t%6.2f\t%6.2f" % (self.Radius[0],self.Radius[1],self.Radius[2])
        msg = msg + "\t%8.3f" % (self.Airmass)
        return msg


# Peak data
class PeakData:
    def __init__ (self, dlista):
        self.Id = dlista[0]
        self.X = float(dlista[1])
        self.Y = float(dlista[2])
        self.pix = float(dlista[3])
        self.mean = float(dlista[4])
        self.dev = float(dlista[5])
        self.med = float(dlista[6])
        self.min = float(dlista[7])
        self.max = float(dlista[8])
        self.fx = float(dlista[9])
        self.fy = float(dlista[10])
        self.FWHM = float(dlista[11])
        self.flux = float(dlista[12])
        self.NX = self.X
        self.NY = self.Y

    def __str__ (self):
        ostr = "%5s\t%7.2f\t%7.2f\t%5d\t%8.2f\t" % (self.Id, self.X, self.Y, self.pix, self.mean)
        ostr = ostr + "%8.2f\t%8.2f\t%8.2f\t%8.2f\t" % (self.dev, self.med, self.min, self.max)
        ostr = ostr + "%7.2f\t%7.2f\t%7.2f\t%10.3f\t" % (self.fx, self.fy, self.FWHM, self.flux)
        ostr = ostr + "%7.2f\t%7.2f" % (self.NX, self.NY)
        return ostr+os.linesep


    def __cmp__ (self, other):
        if self.flux > other.flux:
            return 1
        elif self.flux == other.flux:
            return 0
        else:
            return -1


def rotoTrasla (coords, x0=0.0, y0=0.0, alpha=0.0, halfx=0.0, halfy=0.0):
    x = coords[0]-halfx
    y = coords[1]-halfy
    xn = x0 + x*math.cos(math.radians(alpha))-y*math.sin(math.radians(alpha))
    yn = y0 + x*math.sin(math.radians(alpha))+y*math.cos(math.radians(alpha))
    NX = xn+halfx
    NY = yn+halfy
    return NX,NY

def antiRotoTrasla (coords, x0=0.0, y0=0.0, alpha=0.0, halfx=0.0, halfy=0.0):
    x = coords[0]-halfx-x0
    y = coords[1]-halfy-y0
    xn = x*math.cos(math.radians(alpha))+y*math.sin(math.radians(alpha))
    yn = -x*math.sin(math.radians(alpha))+y*math.cos(math.radians(alpha))
    NX = xn+halfx
    NY = yn+halfy
    return NX,NY


def angleRange (ang):
    while not 0.0 <= ang <= 360.0:
        if ang < 0.0:
            ang = ang + 360.0
        elif ang > 360.0:
            ang = ang - 360.0
    return ang


# Jy -> erg/scm2A
# lambda micron
#def Jy2Ecm2sA (fluden, lambd):
#    return fluden*3e-5/(lambd*1e4)**2

# erg/scm2A -> Jy
# lambda micron
#def Ecm2sA2Jy (fluden, lambd):
#    return 3e41*fluden/(3e14/lambd)**2



""" Utility functions and classes for SRP

Context : SRP
Module  : System.py
Version : 1.0.0
Author  : Stefano Covino
Date    : 02/10/2011
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Usage   : ListOfFitsFiles (file)
            file is either a FITS file or a text file with a list of FITS files. 
            It reports a list of FITS files.

History : (02/10/2011) First version.

"""


import os, os.path
from SRP.SRPFits.IsFits import IsFits


def ListOfFitsFiles (file):
    if os.path.isfile(file):
        if IsFits(file):
            # case with single FITS file in input
            return [file,]
        else:
            with open(file, 'r') as f:
                read_files = f.readlines()
            listfiles = []
            # case with list of FITS files
            for ifts in read_files:
                if IsFits(ifts.split()[0]):
                    listfiles.append(ifts.split()[0].strip())
                else:
                    return None
            return listfiles
    else:    
        # case with input not even file
        return None
        


""" Init file for SRPSystem

Context : SRP
Module  : System
Version : 1.0.5
Author  : Stefano Covino
Date    : 30/08/2013
E-mail  : stefano.covino@brera.inaf.it
URL     : http://www.me.oa-brera.inaf.it/utenti/covino

Usage   : to be imported

History : (24/08/2011) First named version.
        : (02/10/2011) List of Fits files function.
        : (05/02/2012) Solid path.
        : (15/06/2012) New function.
        : (29/08/2013) New function.
        : (30/08/2013) New function.
"""



__all__ = ['CopyGzipped', 'EnsureDir', 'HowOldIsFile', 'ListDirTimeSorted', 
            'ListOfFitsFiles', 'Pipe', 'SRPPath', 'Touch', 'Which']




""" Utility functions and classes for SRP

Context : SRP
Module  : Spectroscopy.py
Version : 1.0.0
Author  : Stefano Covino
Date    : 30/05/2010
E-mail  : stefano.covino@brera.inaf.it
URL:    : http://www.merate.mi.astro.it/utenti/covino

Usage   : to be imported

Remarks : See Morton (Ap. J. Suppl. 77, 119) for the formula used

History : (30/05/2010) First version.

"""

def Air2Vacuum (air):
        if air < 2000:
            return air
        else:
            sigma2 = 1.0/air**2
            fact = 1.0 + 6.4328e-5 + 2.94981e-2/(146.0 - sigma2) + 2.5540e-4/( 41.0 - sigma2)
            return air*fact


""" 

Context : SRP
Module  : Tables
Version : 1.0.1
Author  : Stefano Covino
Date    : 11/02/2013
E-mail  : stefano.covino@brera.inaf.it
URL     : http://www.me.oa-brera.inaf.it/utenti/covino


Usage   : to be imported

Remarks :

History : (08/11/2010) First version.
        : (11/02/2013) CleanStr and SkyStr added.
"""



__all__ = ['CleanStr', 'FindObjects', 'SkyStr']

